#-- encoding: UTF-8

#-- copyright
# OpenProject is a project management system.
# Copyright (C) 2012-2018 the OpenProject Foundation (OPF)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License version 3.
#
# OpenProject is a fork of ChiliProject, which is a fork of Redmine. The copyright follows:
# Copyright (C) 2006-2017 Jean-Philippe Lang
# Copyright (C) 2010-2013 the ChiliProject Team
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
# See docs/COPYRIGHT.rdoc for more details.
#++

class Queries::Projects::Orders::CustomFieldOrder < Queries::BaseOrder
  self.model = Project.all

  validates :custom_field, presence: { message: I18n.t(:'activerecord.errors.messages.does_not_exist') }

  def self.key
    /cf_(\d+)/
  end

  def custom_field
    @custom_field ||= begin
      id = self.class.key.match(attribute)[1]

      ProjectCustomField.visible.find_by_id(id)
    end
  end

  def scope
    super.select(custom_field.order_statements)
  end

  private

  def order
    joined_statement = custom_field.order_statements.map do |statement|
      Arel.sql("#{statement} #{direction}")
    end

    model.order(joined_statement)
  end
end
