#-- encoding: UTF-8
#-- copyright
# OpenProject is a project management system.
# Copyright (C) 2012-2018 the OpenProject Foundation (OPF)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License version 3.
#
# OpenProject is a fork of ChiliProject, which is a fork of Redmine. The copyright follows:
# Copyright (C) 2006-2017 Jean-Philippe Lang
# Copyright (C) 2010-2013 the ChiliProject Team
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
# See docs/COPYRIGHT.rdoc for more details.
#++

require 'api/v3/statuses/status_collection_representer'
require 'api/v3/statuses/status_representer'

module API
  module V3
    module Statuses
      class StatusesAPI < ::API::OpenProjectAPI
        resources :statuses do
          after_validation do
            authorize(:view_work_packages, global: true)
          end

          get do
            StatusCollectionRepresenter.new(Status.all,
                                            api_v3_paths.statuses,
                                            current_user: current_user)
          end

          route_param :id, type: Integer, desc: 'Status ID' do
            helpers do
              # Note that naming the method #status or having
              # a variable named @status colides with grape.
              def work_package_status
                Status.find(params[:id])
              end
            end

            get do
              StatusRepresenter.new(work_package_status, current_user: current_user)
            end
          end
        end
      end
    end
  end
end
