#-- encoding: UTF-8

#-- copyright
# OpenProject is a project management system.
# Copyright (C) 2012-2018 the OpenProject Foundation (OPF)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License version 3.
#
# OpenProject is a fork of ChiliProject, which is a fork of Redmine. The copyright follows:
# Copyright (C) 2006-2017 Jean-Philippe Lang
# Copyright (C) 2010-2013 the ChiliProject Team
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
# See docs/COPYRIGHT.rdoc for more details.
#++

module Bcf::API::V2_1
  class TopicsAPI < ::API::OpenProjectAPI
    resources :topics do
      helpers do
        def topics
          Bcf::Issue.of_project(@project)
        end

        def transform_attributes(attributes)
          wp_attributes = Bcf::Issues::TransformAttributesService
                            .new(@project)
                            .call(attributes)
                            .result

          attributes
            .slice(*Bcf::Issue::SETTABLE_ATTRIBUTES)
            .merge(wp_attributes)
        end

        # In a put request, every non required and non provided
        # parameter needs to be nilled. As we cannot nil type, status and priority
        # as they are required for a work package we use the default values.
        def default_put_params
          {
            index: nil,
            assigned_to: nil,
            description: nil,
            due_date: nil,
            subject: nil,
            type: @project.types.default.first,
            status: Status.default,
            priority: IssuePriority.default
          }
        end
      end

      after_validation do
        authorize :view_linked_issues, context: @project
      end

      get &::Bcf::API::V2_1::Endpoints::Index
             .new(model: Bcf::Issue,
                  api_name: 'Topics',
                  scope: -> { topics })
             .mount

      post &::Bcf::API::V2_1::Endpoints::Create
             .new(model: Bcf::Issue,
                  api_name: 'Topics',
                  params_modifier: ->(attributes) {
                    transform_attributes(attributes)
                      .merge(project: @project)
                  })
             .mount

      route_param :topic_uuid, regexp: /\A[a-f0-9\-]+\z/ do
        after_validation do
          @issue = topics.find_by_uuid!(params[:topic_uuid])
        end

        get &::Bcf::API::V2_1::Endpoints::Show
              .new(model: Bcf::Issue,
                   api_name: 'Topics')
              .mount

        put &::Bcf::API::V2_1::Endpoints::Update
               .new(model: Bcf::Issue,
                    api_name: 'Topics',
                    params_modifier: ->(attributes) {
                      transform_attributes(attributes)
                        .reverse_merge(default_put_params)
                    })
               .mount

        delete &::Bcf::API::V2_1::Endpoints::Delete
                  .new(model: Bcf::Issue,
                       api_name: 'Topics')
                  .mount

        mount Bcf::API::V2_1::Viewpoints::API
      end
    end
  end
end
