#-- copyright
# OpenProject is a project management system.
# Copyright (C) 2012-2018 the OpenProject Foundation (OPF)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License version 3.
#
# OpenProject is a fork of ChiliProject, which is a fork of Redmine. The copyright follows:
# Copyright (C) 2006-2017 Jean-Philippe Lang
# Copyright (C) 2010-2013 the ChiliProject Team
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
# See docs/COPYRIGHT.rdoc for more details.
#++

require 'spec_helper'
require 'features/work_packages/work_packages_page'

describe 'Work package details toolbar', js: true, selenium: true do
  let(:project) { FactoryBot.create :project_with_types, public: true }
  let!(:work_package) { FactoryBot.create :work_package, project: project }
  let(:work_packages_page) { WorkPackagesPage.new(project) }

  describe 'toggle watch state' do
    let(:user) { FactoryBot.create :admin }
    before do
      login_as(user)
      work_packages_page.visit_index(work_package)
    end

    it 'toggles the watch state' do
      expect(work_package.watcher_users).not_to include(user)
      expect(page).to have_selector('.work-packages--details-toolbar button', text: 'Watch')
      within '.work-packages--details-toolbar' do
        click_button 'Watch'
      end

      expect(page).to have_selector('.work-packages--details-toolbar button', text: 'Unwatch')

      expect(work_package.reload.watcher_users).to include(user)
      expect(page).to have_selector('.work-packages--details-toolbar button', text: 'Unwatch')
    end
  end
end
